package freeshow;

import java.awt.*;
import java.awt.image.*;
import java.awt.font.*;
import java.awt.event.*;
import java.text.*;
import java.util.*;
import javax.imageio.*;
import java.lang.reflect.*;

public class FScanvas extends Canvas {
    Image cimage;
    public boolean showImage = true;
    public boolean showText = true;
    public boolean playSound = true;
    Image muteicon;
    Image notexticon;
    Image pauseicon;
    Robot robot;

    public FScanvas() {
        // load icon images upon initialization
        ClassLoader cl = this.getClass().getClassLoader();
         try {
            muteicon = ImageIO.read(cl.getResource("icon-muted.png"));
            notexticon = ImageIO.read(cl.getResource("icon-notext.png"));
            pauseicon = ImageIO.read(cl.getResource("icon-paused.png"));
        } catch(Exception e) {
            System.out.println(e.getMessage());
        }

        // create robot for generating fake key events
        try {
            robot = new Robot();
        }
        catch(Exception e) {
            robot = null;
            System.out.println(e.getMessage());
        }


    }
    private void drawText(FStext fst, Graphics2D g2, int cwidth,
                          int cheight) {
        // *********************
        // Draw the text string
        // *********************
        if (fst.text == null || fst.text.length() == 0) {
            return;
        }
        // start at lower edge of image
        float y = (float) (cheight * (1.0f - fst.ytxt));
        // split text on line breaks
        String[]strarr = fst.text.split("\\\\n");
        // System.out.println("No of substrings: " + Array.getLength(strarr));
        // loop over substrings
        String dstr;
        for (int i = 0; i < Array.getLength(strarr); i++) {
            dstr = strarr[i];
            AttributedString attstring = new AttributedString(dstr);
            // set text attributes
            attstring.addAttribute(TextAttribute.FOREGROUND,
                                   new Color(fst.txtcolor[0],
                                             fst.txtcolor[1],
                                             fst.txtcolor[2]));
            //Font font = new Font("serif", Font.PLAIN, fst.fontsize);
            Font font = new Font(fst.font, fst.style, fst.fontsize);
            attstring.addAttribute(TextAttribute.FONT, font);

            LineBreakMeasurer linebreaker =
                new LineBreakMeasurer(attstring.getIterator(),
                                      g2.getFontRenderContext());


            while (linebreaker.getPosition() < dstr.length()) {
                TextLayout tl =
                    linebreaker.nextLayout(cwidth * fst.txtwidth);

                y += tl.getAscent();
                //System.out.println("Printing " + dstr + " " + cwidth * fst.xtxt + " " +y);
                tl.draw(g2, cwidth * fst.xtxt, y);
                y += tl.getDescent() + tl.getLeading();
            }
        }
    }

    public void render(Graphics g, Freeslide slide) {
        int[] col;
        int x0, y0;
        int pheight, pwidth;
        int cheight, cwidth;
        double scale;

        // cast graphics device to Graphics2D
        Graphics2D g2 = (Graphics2D) g;
        // Get canvas dimension
        cheight = g2.getDeviceConfiguration().getBounds().height;
        cwidth = g2.getDeviceConfiguration().getBounds().width;

        // Clear canvas to background color
        //col = Xpm.NameToRGB3 (slide.bgcolor);
        g2.setBackground(new
                         Color(slide.bgcolor[0], slide.bgcolor[1],
                               slide.bgcolor[2]));
        g2.clearRect(0, 0, cwidth, cheight);

        // Start with image
        if (slide.imagefile != null && showImage == true) {
            // Load image
            if (slide.isPreloaded == false) {
                slide.preload();
            }
            // Calculate image size and placement
            // We want one quarter of the height to be for text
            pheight = (int) (cheight * slide.ImageVertSize);
            // Then we know the scale:
            scale = pheight / (1.0 * slide.img.getHeight(this));
            // From scale we calculate painting width
            pwidth = (int) (slide.img.getWidth(this) * scale);
            // If the image became to wide, rescale after width
            if (pwidth > cwidth) {
                pwidth = cwidth;
                scale = pwidth / (1.0 * slide.img.getWidth(this));
                pheight = (int) (slide.img.getHeight(this) * scale);
            }
            // Centre image horizontally
            if (slide.ImageJustification.equals("right")) {
                x0 = cwidth - pwidth;
            } else if (slide.ImageJustification.equals("left")) {
                x0 = 0;
            } else {
                x0 = (int) ((cwidth - pwidth) / 2.0);
            }
            y0 = 0;

            try {
                g2.drawImage(slide.img, x0, y0, pwidth, pheight, this);
            }
            catch(Exception e) {
                System.out.println(e.getMessage());
            }
            // Free image in order to save memory.
            slide.img = null;
            slide.isPreloaded = false;
            //System.out.println(Runtime.getRuntime().freeMemory());
        }
        // draw each string
        if (showText == true && slide.libretto.size() > 0) {
            Iterator it = slide.libretto.iterator();
            while (it.hasNext()) {
                drawText((FStext) it.next(), g2, cwidth, cheight);
            }
        }
        // Finally, paint icons if mute/notext
        x0 = 0;
        y0 = 0;
        pwidth = 0;
        if (playSound == false) {
            // set scale so that 1/10th of height is occupied by icon
            pheight = (int) (0.05 * cheight);
            scale = pheight / (1.0 * muteicon.getHeight(this));
            pwidth = (int) (muteicon.getWidth(this) * scale);
            try {
                g2.drawImage(muteicon, x0, y0, pwidth, pheight, this);
            }
            catch(Exception e) {
                System.out.println(e.getMessage());
            }
        }
        if (showText == false) {
            x0 += pwidth;
            y0 = 0;
            // set scale so that 1/10th of height is occupied by icon
            pheight = (int) (0.04 * cheight);
            scale = pheight / (1.0 * notexticon.getHeight(this));
            pwidth = (int) (notexticon.getWidth(this) * scale);
            try {
                g2.drawImage(notexticon, x0, y0, pwidth, pheight, this);
            }
            catch(Exception e) {
                System.out.println(e.getMessage());
            }

        }

        if (slide.isPaused == true) {
            x0 += pwidth;
            y0 = 0;
            // set scale so that 1/10th of height is occupied by icon
            pheight = (int) (0.05 * cheight);
            scale = pheight / (1.0 * pauseicon.getHeight(this));
            pwidth = (int) (pauseicon.getWidth(this) * scale);
            try {
                g2.drawImage(pauseicon, x0, y0, pwidth, pheight, this);
            }
            catch(Exception e) {
                System.out.println(e.getMessage());
            }

        }
        // Generate a keypress in order to stop screensaver
        if (robot != null) {
            robot.keyPress(KeyEvent.VK_SHIFT);
            robot.keyRelease(KeyEvent.VK_SHIFT);
        }

    };


    public void playslide(Freeslide s, BufferStrategy strategy) {
        // Pause if muted
        if (playSound == false) {
            s.isPaused = true;
        } else {
            s.isPaused = false;
        }
        s.isStopped = false;
        s.isDone = false;
        Graphics g = strategy.getDrawGraphics();
        render(g, s);
        g.dispose();
        strategy.show();
        s.playAudio(playSound);
    }
    public void redraw(Freeslide s, BufferStrategy strategy) {
        Graphics g = strategy.getDrawGraphics();
        render(g, s);
        g.dispose();
        strategy.show();
    }
}
