package freeshow;

import java.io.*;
import org.jdom.*;
import org.jdom.input.SAXBuilder;
import java.util.*;
import java.net.URL;
import org.jdom.output.XMLOutputter;
import org.jdom.output.Format;

public class FSdata {

    private LinkedList list = new LinkedList();
    private Freeslide dr;
    private String basepath;
    private String globalsound;
    private String sep;

    public FSdata(String fname) {
        /* hack if fname is net address */
        if(fname.matches("http://.*") || fname.matches("ftp://.*") ) {
            sep = "/";
            int itmp;
            itmp = fname.lastIndexOf('/');
            basepath = fname.substring(0,itmp);
            System.out.println(basepath);
        }
        else { 
            sep = File.separator;
            basepath = new File(fname).getParent();
        }
        this.parseInput(fname);
    } public void parseInput(String fname) {
        try {
            if (fname.length() == 0) {
                System.out.println("Usage: java -jar freeshow-*.jar URL");
                return;
            }

            SAXBuilder builder = new SAXBuilder(true);
            try {
                Document doc;
                if(fname.matches("http://.*") || fname.matches("ftp://.*")){
                    doc = builder.build(new URL(fname));
                }
                else {
                    doc = builder.build(fname);
                }
                // If there are no well-formedness or validity errors,
                // then no exception is thrown.
                //System.out.println (fname + " is valid.");
                parseSlides(doc.getRootElement());
            }
            catch(JDOMException e) {
                System.out.println("Error: " + fname + " is not valid.");
                System.out.println(e.getMessage());
                System.exit(0);
            }
            catch(IOException e) {
                System.out.println("Could not check " + fname);
                System.out.println(" because " + e.getMessage());
                System.exit(0);
            }
        }
        catch(Exception e) {
            e.printStackTrace();
        }

    }

    public void parseSlides(Element o) {
        String au = "", te = "", im = "", jus = "";
        float x = 0.0f, y = 0.0f, w = 0.0f, sz = 0.0f, imsz = 0.0f;
        String bgc = "", txc = "", txcti = "";
        String font = "", fontstyle = "";
        int t = 0, tsz = 0, tszti = 0;
        int[] bgcol = { 1, 1, 1 }, txtcol = {
        0, 0, 0}, txtcolti = {
        0, 0, 0};
        Element e2;
        LinkedList textlist = null;
        FStext fstext;
        Freeslide newslide;

        globalsound=o.getAttributeValue("globalsound");
        List children = o.getChildren("slide");
        Iterator iterator = children.iterator();
        /* loop over slides */
        while (iterator.hasNext()) {
            try {
                Element tmpslide = (Element) iterator.next();
                /* slide attributes */
                t = tmpslide.getAttribute("mintime").getIntValue();
                bgc = tmpslide.getAttributeValue("backgroundcolor");
                bgcol = Xpm.NameToRGB3(bgc);
                if (bgcol == null) {
                    bgcol = new int[] {
                    1, 1, 1};
                };

                /* slide image */
                im = tmpslide.getChildText("image");
                if (im != null && new File(im).isAbsolute() == false
                    && basepath != null && !im.matches("http://.*")  && !im.matches("ftp://.*")) {
                    im = basepath + sep + im;
                }
                /* image attributes */
                e2 = tmpslide.getChild("image");
                if (e2 != null) {
                    imsz = e2.getAttribute("verticalsize").getFloatValue();
                    if (imsz <= 0.0 || imsz > 1.0) {
                        imsz = 0.75f;
                    }
                    jus = e2.getAttributeValue("justification");
                }
                /* sound file */
                au = tmpslide.getChildText("sound");
                if (au != null && new File(au).isAbsolute() == false
                    && basepath != null && !au.matches("http://.*")  && !au.matches("ftp://.*")) {
                    au = basepath + sep + au;
                    //au = new File(au).getAbsolutePath();
                }
                // slide text

                List text = tmpslide.getChildren("text");
                textlist = new LinkedList();

                Iterator ti = text.iterator();
                /* loop over slide texts */
                while (ti.hasNext()) {
                    try {
                        Element tmptxt = (Element) ti.next();
                        te = tmptxt.getText();
                        if (tmptxt != null) {
                            x = tmptxt.getAttribute("xcoord").
                                getFloatValue();
                            if (x <= 0.0 || x > 1.0) {
                                x = 0.0f;
                            }
                            y = tmptxt.getAttribute("ycoord").
                                getFloatValue();
                            if (y <= 0.0 || y > 1.0) {
                                y = 0.75f;
                            }
                            w = tmptxt.getAttribute("textwidth").
                                getFloatValue();
                            if (w <= 0.0 || w > 1.0) {
                                w = 1.0f;
                            }
                            txc = tmptxt.getAttributeValue("textcolor");
                            txtcol = Xpm.NameToRGB3(txc);
                            if (txtcol == null) {
                                txtcol = new int[] {
                                0, 0, 0};
                            };
                            font = tmptxt.getAttributeValue("font");
                            fontstyle = tmptxt.getAttributeValue("style");
                            tsz =
                                tmptxt.getAttribute("fontsize").
                                getIntValue();
                            if (tsz < 8 || tsz > 100) {
                                tsz = 24;
                            }
                        }
                    }
                    catch(Exception e) {
                        System.out.println(e.getMessage());
                    }
                    fstext = new FStext(te);
                    fstext.font = font;
                    /* parse font style */
                    if(fontstyle.matches("bold")){
                        fstext.style = java.awt.Font.BOLD;
                    }
                    else if(fontstyle.matches("italic")){
                        fstext.style = java.awt.Font.ITALIC;
                    }
                    else {
                        fstext.style = java.awt.Font.PLAIN;
                    }

                    fstext.fontsize = tsz;
                    fstext.xtxt = x;
                    fstext.ytxt = y;
                    fstext.txtwidth = w;
                    fstext.txtcolor = txtcol;
                    try {
                        textlist.addLast(fstext);
                    }
                    catch(Exception e) {
                        e.printStackTrace();
                    }

                }

            }
            catch(Exception e) {
                System.out.println(e.getMessage());
            }

            /* create new slide */
            newslide = new Freeslide(textlist, im, au);
            newslide.bgcolor = bgcol;
            newslide.duration = t;
            newslide.ImageVertSize = imsz;
            newslide.ImageJustification = jus;


            /* add slide to list */
            try {
                list.addLast(newslide);
            }
            catch(Exception e) {
                e.printStackTrace();
            }
        }
    }

    public LinkedList getList() {
        return list;
    }
    public String getGlobalSound() {
        return globalsound;
    }

    // a first stab at saving the slideshow to a file,
    // for later use...
    /*
       public void save(String fname) {
       int i;
       Element el = new Element("slide");
       Element fs = new Element("slideshow");
       Freeslide s;

       for (i = 0; i < list.size(); i++) {
       s = (freeshow.Freeslide) list.get(i);
       el = new Element("slide");
       if (s.text != null && s.text.length() > 0) {
       Element text = new Element("text");
       text.addContent(s.text);
       System.out.println(s.text);
       el.addContent(text);
       }
       if (s.imagefile != null) {
       Element image = new Element("image");
       image.addContent(s.imagefile);
       el.addContent(image);
       }
       if (s.soundfile != null) {
       Element sound = new Element("sound");
       sound.addContent(s.soundfile);
       el.addContent(sound);
       }
       fs.addContent(el);
       }
       // create document
       Document d = new Document(fs);
       // save it to a file:
       XMLOutputter out = new XMLOutputter();
       out.setFormat(org.jdom.output.Format.getPrettyFormat());
       try {
       java.io.FileWriter writer = new java.io.FileWriter(fname);
       out.output(d, writer);
       writer.flush();
       writer.close();
       }
       catch(Exception e) {
       System.out.println(e.getMessage());
       }
       }
     */

}
