package freeshow;

import java.awt.*;
import java.util.*;
import java.awt.event.*;
import java.awt.image.*;

/**
 * <p>Title: freeshow</p>
 * <p>Description: slideshow application</p>
 * <p>Copyright: Copyright (c) 2006</p>
 * <p>Company: </p>
 * @author Mikael Borg
 * @version 1.0
 */

public class Freeshow extends Frame {
    FScanvas c;
    LinkedList list;
    String globalSnd;
    Freeslide globalslide; 
    // Freeslide s=new Freeslide();
    boolean reachedEnd = false;
    Rectangle bounds;
    static GraphicsDevice device = null;
    static GraphicsEnvironment env;
    Freeslide current;
    int currentno;
    BufferStrategy strategy;


    // Freeshow constructor: adds a FScanvas to 
    // main window, reads in slideshow data using FSdata
    public Freeshow(String infile) throws HeadlessException {
        super(device.getDefaultConfiguration());
        this.setUndecorated(true);
        FSdata d = new FSdata(infile);
        device.setFullScreenWindow(this);

        if (!this.isFocusOwner()) {
            this.setFocusable(true);
        }
        KeyMonitor monitor = new KeyMonitor(this);
         addKeyListener(monitor);
         list = d.getList();
         globalSnd=d.getGlobalSound();
         if(globalSnd.length()>0) {
             globalslide = new Freeslide(null, "", globalSnd);
             globalslide.playAudio(true);
         }
         current = (freeshow.Freeslide) list.getFirst();
         bounds = this.getBounds();
         this.setVisible(true);
         createBufferStrategy(2);
         strategy = getBufferStrategy();
         this.c = new FScanvas();
         this.add("Center", c);
        // test saving
        // d.save("testsave.xml");

    }
    // main method: initialize graphics and start show
        public static void main(String[]args) throws HeadlessException
    {
        try {
            env = GraphicsEnvironment.getLocalGraphicsEnvironment();
            device = env.getDefaultScreenDevice();
        } catch(Exception e) {
        }

        String datafile;
        if (args.length == 0) 
            datafile="index.xml";
        else
            datafile=args[0];
        
        Freeshow fs = new Freeshow(datafile);
        fs.setBackground(Color.WHITE);
        fs.Play(0);
        System.exit(0);

    }

    // Play show, starting w/ startno
    private void Play(int startno) {

        if (startno < 0) {
            startno = 0;
        }
        // if another slide is currently playing, wait until it is done
        if (current != null) {
            while (current.isPlaying == true) {
                try {
                    this.wait(100);
                }
                catch(Exception e) {
                }
            }
        }

        //System.out.println ("Entering play loop");
        for (currentno = startno; currentno < list.size(); currentno++) {
            try {
                if (currentno < 0 || currentno > list.size()) {
                    currentno = 0;
                }
                current = (Freeslide) list.get(currentno);
            }
            catch(Exception e) {
                System.out.println(e.getMessage());
            }
            //System.out.println ("Playing no " + currentno);
            c.playslide(current, strategy);
            while (current.isDone == false) {
                try {
                    this.wait(100);
                }
                catch(Exception e) {
                }
            }
        }
    }

    // Keyadapter performs actions when buttons are pressed
    class KeyMonitor extends KeyAdapter {
        Freeshow display;

         KeyMonitor(Freeshow display) {
            this.display = display;
        } public void keyPressed(KeyEvent e) {
            int kc;
            try {
            kc = e.getKeyCode();
            //System.out.println(e.toString());
            }
                    catch(Exception ex) {
                        kc = 0;
                    };

            if (kc == KeyEvent.VK_ESCAPE) {
                System.exit(0);
            } else if (kc == KeyEvent.VK_RIGHT) {
                if (currentno < list.size() - 1) {
                    current.isStopped = true;
                    try {
                        wait(100);
                    }
                    catch(Exception ex) {
                    };
                }
            }  else if (kc == KeyEvent.VK_PAGE_DOWN) {
                if (currentno < list.size() - 1) {
                    current.isStopped = true;
                    try {
                        wait(100);
                    }
                    catch(Exception ex) {
                    };
                    if (currentno <= list.size() - 12) {
                            currentno += 9;
                    } else  {currentno = list.size()-2;} 
                }
            }  else if (kc == KeyEvent.VK_PAGE_UP) {
                    current.isStopped = true;
                    try {
                        wait(100);
                    }
                    catch(Exception ex) {
                    };
                    currentno -= 11;
            } else if (kc == KeyEvent.VK_LEFT) {
                current.isStopped = true;
                try {
                    wait(100);
                }
                catch(Exception ex) {
                };
                if (currentno > 0) {
                    currentno -= 2;
                } else if (currentno == 0) {
                    currentno--;
                }
            } else if (kc == KeyEvent.VK_DOWN) {
                current.isStopped = true;
                try {
                    wait(100);
                }
                catch(Exception ex) {
                };
                if (currentno > 0) {
                    currentno--;
                }
            } else if (kc == KeyEvent.VK_HOME) {
                current.isStopped = true;
                //globalslide.isStopped = true;
                //System.out.println("home pressed\n");

                try {
                    wait(100);
                }
                catch(Exception ex) {
                System.out.println(ex.getMessage());
                };
                //globalslide.playAudio(true);
                currentno = -1;
            } else if (kc == KeyEvent.VK_END) {
                current.isStopped = true;
                try {
                    wait(100);
                }
                catch(Exception ex) {
                };
                currentno = list.size() - 2;
            } else if (kc == KeyEvent.VK_P || kc == KeyEvent.VK_SPACE) {
                if (current.isPaused == true) {
                    current.isPaused = false;
                    globalslide.isPaused = false;
                } else {
                    current.isPaused = true;
                    globalslide.isPaused = true;
                }
                c.redraw(current, strategy);
            } else if (kc == KeyEvent.VK_T) {
                if (c.showText == true) {
                    c.showText = false;
                } else {
                    c.showText = true;
                }
                c.redraw(current, strategy);
            } else if (kc == KeyEvent.VK_S) {
                if (c.playSound == true) {
                    c.playSound = false;
                } else {
                    c.playSound = true;
                }
                current.isStopped = true;
                try {
                    wait(100);
                }
                catch(Exception ex) {
                };
                if (currentno >= 0) {
                    currentno--;
                }
                c.redraw(current, strategy);
                //current.isStopped = true;
            } else if (kc == KeyEvent.VK_I) {
                if (c.showImage == true) {
                    c.showImage = false;
                } else {
                    c.showImage = true;
                }
                c.redraw(current, strategy);
            } else {}
            //System.out.println (kc);
        }





    }


}
