package freeshow;
import java.awt.*;
import java.io.*;
import java.util.*;
import java.net.URL;
import javax.sound.sampled.*;
import javax.imageio.*;
import java.lang.System;

import javazoom.spi.PropertiesContainer;

class Freeslide {
    public String imagefile;    /* slide image file */
    public LinkedList libretto = new LinkedList();      /* list with slide strings */
    public String soundfile;    /* slide sound file */
    public float ImageVertSize; /* vertical image size, in percent of screen height */
    public String ImageJustification;   /* horizontal image justification */
    public int[] bgcolor;       /* background color */
    public int duration;        /* minimum slide playing time, in seconds */
    public Image img;
    public boolean isPreloaded;
    public boolean isPaused;
    public boolean isStopped;
    public boolean isPlaying;
    public boolean isDone;
    public Dimension csize;
    AudioFormat audioFormat;
    AudioInputStream audioInputStream;
    AudioInputStream din = null;
    SourceDataLine sourceDataLine;


    public Freeslide(LinkedList l, String im, String so) {
        libretto = l;
        imagefile = im;
        soundfile = so;
        isPreloaded = false;
        isPaused = false;
        isStopped = false;
        isPlaying = false;
        isDone = false;
    } public void preload() {
        try {
            //System.out.println ("Loading " + imagefile);
            if(imagefile.matches("http://.*") || imagefile.matches("ftp://.*")){
                img = ImageIO.read(new URL(imagefile));
            }
            else {
                img = ImageIO.read(new File(imagefile));
            }
            isPreloaded = true;
        } catch(Exception e) {
            System.out.println ("File " + imagefile);
            System.out.println(e.getMessage());
        }

    };
    public void playAudio(boolean playSound) {
        //ClassLoader cl = this.getClass ().getClassLoader ();
        if (soundfile != null && playSound == true) {
            try {

                try {
                    //  System.out.println(AudioSystem.getAudioFileFormat(soundfile).getType());
                    if(soundfile.matches("http://.*") || soundfile.matches("ftp://.*")){
                        audioInputStream =
                            AudioSystem.
                            getAudioInputStream(new URL(soundfile));
                    }
                    else {
                        audioInputStream =
                            AudioSystem.
                            getAudioInputStream(new File(soundfile));
                    }
                }
                catch(Exception e) {
                    System.out.println("Could not open audio file " +
                            soundfile);
                    e.printStackTrace();
                    System.exit(0);
                }               //end catch

                audioFormat = audioInputStream.getFormat();
                if (audioFormat.getEncoding() !=
                        AudioFormat.Encoding.PCM_SIGNED
                        && audioFormat.getEncoding() !=
                        AudioFormat.Encoding.PCM_UNSIGNED) {
                    //System.out.println(audioFormat);
                    AudioFormat decodedFormat =
                        new AudioFormat(AudioFormat.Encoding.PCM_SIGNED,
                                audioFormat.getSampleRate(),
                                16,
                                audioFormat.getChannels(),
                                audioFormat.getChannels() * 2,
                                audioFormat.getSampleRate(),
                                false);
                    // Get AudioInputStream that will be decoded by underlying VorbisSPI
                    din =
                        AudioSystem.getAudioInputStream(decodedFormat,
                                audioInputStream);
                    sourceDataLine =
                        (SourceDataLine) getSourceDataLine(decodedFormat);
                } else {
                    din = audioInputStream;
                    sourceDataLine = (SourceDataLine) getSourceDataLine(audioFormat);
                }

                //DataLine.Info dataLineInfo =
                //  new DataLine.Info (SourceDataLine.class, audioFormat);


                //Create a thread to play back the data and
                // start it running.  It will run until the
                // end of file, or the Stop button is
                // clicked, whichever occurs first.
                // Because of the data buffers involved,
                // there will normally be a delay between
                // the click on the Stop button and the
                // actual termination of playback.
            }
            catch(Exception e) {
                e.printStackTrace();
                System.exit(0);
            }                   //end catch
            if(sourceDataLine != null){
                new PlayThread().start();}
            else {
                playAudio(false);
            }
        }
        // if no sound file for this slide, wait specified time
        else {
            isPlaying = true;
            long tstart = System.currentTimeMillis();
            long tstop = tstart + duration * 1000;
            while ((System.currentTimeMillis() < tstop || isPaused == true)
                    && isStopped == false) {
                try {
                    this.wait(100);
                }
                catch(Exception e) {
                }
                    }
            isDone = true;
            isPlaying = false;
        }

    }                           //end playAudio



    //=============================================//
    //Inner class to play back the data from the
    // audio file.
    class PlayThread extends Thread {
        /* The size of tempBuffer is important for sound quality, if it is too 
         * small, the sound becomes choppy. If it is too large, there is a 
         * long delay when pressing e.g. the pause button. The current size
         * was selected after testing on Windows 2000 computers, that seem to
         * need a larger buffer size than real operating systems.
         */
        byte tempBuffer[] = new byte[8192];

        public void run() {
            long tstart, tstop;

            tstart = System.currentTimeMillis();
            tstop = tstart + duration * 1000;
            try {
                isDone = false;
                isPlaying = true;
                //sourceDataLine.open (audioFormat);
                sourceDataLine.start();

                int cnt;
                //Keep looping until the input read method
                // returns -1 for empty stream or the
                // user clicks the Stop button causing
                // stopPlayback to switch from false to
                // true.
                while ((cnt =
                            din.read(tempBuffer, 0,
                                tempBuffer.length)) != -1
                        && isStopped == false) {
                    while (isPaused == true) {
                        this.sleep(100);
                    };
                    if (cnt > 0) {
                        //Write data to the internal buffer of
                        // the data line where it will be
                        // delivered to the speaker.
                        sourceDataLine.write(tempBuffer, 0, cnt);
                    }           //end if
                        }               //end while

                //Block and wait for internal buffer of the
                // data line to empty.
                if (isStopped == false) {
                    sourceDataLine.drain();
                }

                sourceDataLine.close();
                while (System.currentTimeMillis() < tstop
                        && isStopped == false) {
                    this.sleep(100);
                        }
                isDone = true;
                // free memory
                audioFormat=null;
                audioInputStream=null;
                din = null;
                sourceDataLine = null;
                this.sleep(100);
                isPlaying = false;

            }
            catch(Exception e) {
                e.printStackTrace();
                System.exit(0);
            }                   //end catch
        }                       //end run
    }                           //end inner class PlayThread

    private SourceDataLine getSourceDataLine(AudioFormat audioFormat) 
    {
        SourceDataLine res = null;
        DataLine.Info info =
            new DataLine.Info(SourceDataLine.class, audioFormat);
        try {
            res = (SourceDataLine) AudioSystem.getLine(info);
            res.open(audioFormat);
        }
        catch (javax.sound.sampled.LineUnavailableException e){
            System.out.println("Could not open audio output\n");
            res = null;
        }

        return res;
    }
}
